/***************************************************************************
 *   Copyright (C) 2007  Antony Martin <martin.antony@yahoo.fr>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 10 janvier 2007
   derniere modification le 28/10/2007 */

#ifndef __SCECBUFFERS_H__
#define __SCECBUFFERS_H__

#include <SCE/utils/SCEList.h>

/* constantes de definition des types de sommets. (attrib)
   la constante SCE_POSITION peut valoire n'importe quelle valeur entiere
   positive, les autre doivent imperativement valoires PREC+1
   ou PREC represente la constante precedemment definie. */
#define SCE_POSITION 1
#define SCE_COLOR 2
#define SCE_NORMAL 3

/* coordonnees de texture */
#define SCE_TEXCOORD0 4
#define SCE_TEXCOORD1 5
#define SCE_TEXCOORD2 6
#define SCE_TEXCOORD3 7
#define SCE_TEXCOORD4 8
#define SCE_TEXCOORD5 9
#define SCE_TEXCOORD6 10
#define SCE_TEXCOORD7 11
/* s'etend jusqu'au maximum supporte par ma machine, cf CBuffers.c */

/* attributs de sommet */
#define SCE_ATTRIB0 100
#define SCE_ATTRIB1 101
#define SCE_ATTRIB2 102
#define SCE_ATTRIB3 103
#define SCE_ATTRIB4 104
#define SCE_ATTRIB5 105
#define SCE_ATTRIB6 106
#define SCE_ATTRIB7 107
#define SCE_ATTRIB8 108
#define SCE_ATTRIB9 109
#define SCE_ATTRIB10 110
#define SCE_ATTRIB11 111
#define SCE_ATTRIB12 112
#define SCE_ATTRIB13 113
#define SCE_ATTRIB14 114
#define SCE_ATTRIB15 115
/* s'etend jusqu'au maximum supporte par ma machine, cf CBuffers.c */


typedef void (*SCE_FSetDeclaration)(void);

/* declaration d'un type de sommet
   divers information sur l'utilisation d'un tampon */
typedef struct sce_cvertexdeclaration SCE_CVertexDeclaration;
struct sce_cvertexdeclaration
{
    SCE_FSetDeclaration set; /* fonction d'envoi de la declaration a opengl */
    int active;              /* sera utilise ou pas */
    unsigned int attrib;     /* type du sommet (position, normale, ...) */
    SCEenum type;            /* type des donnees (int, float, ...) */
    GLint size;              /* nombre de composantes des vecteurs */
    GLint first;             /* position du premier octet de donnes */
    GLsizei stride;          /* marge entre chaque sommet (en bytes) */
    int user;         /* indique si la declaration appartient a l'utilisateur */
};


/* donnees tampon
   enregistre une sequence de donnees pour le remplissage d'un tampon */
typedef struct sce_cbufferdata SCE_CBufferData;
struct sce_cbufferdata
{
    void *data;       /* donnees */
    size_t data_size; /* taille des donnees */
    size_t first;     /* indice de stockage des donnees dans leur tampon */
    int user;         /* indique si les donnees appartiennent a l'utilisateur */
};


/* objet tampon de sommets opengl */
typedef struct sce_cvertexbuffer SCE_CVertexBuffer;
struct sce_cvertexbuffer
{
    GLuint id;       /* id opengl */
    SCE_SList *data; /* donnees du tampon (BufferData) */
    SCE_SList *decs; /* declaration des sommets (VertexDeclaration) */

    /* taille additionnee des precedents stockages de donnees
       au final, size = la taille totale du tampon */
    size_t size;
};


/* objet tampon d'indices opengl */
typedef struct sce_cindexbuffer SCE_CIndexBuffer;
struct sce_cindexbuffer
{
    GLuint id;       /* id opengl */
    SCE_SList *data; /* donnees du tampon */

    /* taille additionnee des precedents stockage de donnees
       au final, size = la taille total du tampon */
    size_t size;
};

/* defini le buffer actif */
void SCE_CBindVertexBuffer (SCE_CVertexBuffer*);
void SCE_CBindIndexBuffer (SCE_CIndexBuffer*);

void SCE_CInitVertexDeclaration (SCE_CVertexDeclaration*);
SCE_CVertexDeclaration* SCE_CCreateVertexDeclaration (void);
void SCE_CDeleteVertexDeclaration (void*);

void SCE_CInitVertexBuffer (SCE_CVertexBuffer*);
void SCE_CInitIndexBuffer (SCE_CIndexBuffer*);

void SCE_CInitBufferData (SCE_CBufferData*);
SCE_CBufferData* SCE_CCreateBufferData (void);
void SCE_CDeleteBufferData (void*);

/* cree un tampon de sommets */
SCE_CVertexBuffer* SCE_CCreateVertexBuffer (void);
/* cree un tampon d'indices */
SCE_CIndexBuffer* SCE_CCreateIndexBuffer (void);

/* supprime un tampon de sommets */
void SCE_CDeleteVertexBuffer (SCE_CVertexBuffer*);
void SCE_CDeleteVertexBuffer_ (void);
/* supprime un tampon d'indices */
void SCE_CDeleteIndexBuffer (SCE_CIndexBuffer*);
void SCE_CDeleteIndexBuffer_ (void);

/* vide les buffers de leurs donnees */
void SCE_CClearVertexBuffer (SCE_CVertexBuffer*);
void SCE_CClearVertexBuffer_ (void);
void SCE_CClearIndexBuffer (SCE_CIndexBuffer*);
void SCE_CClearIndexBuffer_ (void);

/* envoie des donnees a un tampon de sommets */
int SCE_CAddVertexBufferData (SCE_CVertexBuffer*, size_t, void*);
int SCE_CAddVertexBufferData_ (size_t, void*);
/* idem, mais duplique les donnees */
int SCE_CAddVertexBufferDataDup (SCE_CVertexBuffer*, size_t, void*);
int SCE_CAddVertexBufferDataDup_ (size_t, void*);

/* envoie des donnees a un tampon d'indices */
int SCE_CAddIndexBufferData (SCE_CIndexBuffer*, size_t, void*);
int SCE_CAddIndexBufferData_ (size_t, void*);
/* idem, mais duplique les donnees */
int SCE_CAddIndexBufferDataDup (SCE_CIndexBuffer*, size_t, void*);
int SCE_CAddIndexBufferDataDup_ (size_t, void*);

/* renvoie la taille additionnee de toutes
   les donnees stockees dans le tampon de sommets.
   cette taille est egale au premier emplacement
   disponible pour stocker des donnees dans le tampon */
size_t SCE_CGetVertexBufferFirst (SCE_CVertexBuffer*);

/* assigne le pointeur de fonction a la declaration */
void SCE_CBuildVertexDeclaration (SCE_CVertexDeclaration*);

/* envoie des declarations de vertex a un tampon de sommets */
int SCE_CAddVertexDeclaration (SCE_CVertexBuffer*, SCE_CVertexDeclaration*);
int SCE_CAddVertexDeclaration_ (SCE_CVertexDeclaration*);
/* idem, mais duplique la structure */
int SCE_CAddVertexDeclarationDup (SCE_CVertexBuffer*, SCE_CVertexDeclaration*);
int SCE_CAddVertexDeclarationDup_ (SCE_CVertexDeclaration*);

/* construit un tampon de sommets, envoie les donnees a opengl */
void SCE_CBuildVertexBuffer (SCE_CVertexBuffer*, SCEenum);
void SCE_CBuildVertexBuffer_ (SCEenum);
/* construit un tampon d'indices, envoie les donnees a opengl */
void SCE_CBuildIndexBuffer (SCE_CIndexBuffer*, SCEenum);
void SCE_CBuildIndexBuffer_ (SCEenum);

/* verouille un tampon de sommets
   et renvoie l'adresse memoire de la zone de stockage du tampon */
void* SCE_CLockVertexBuffer (SCE_CVertexBuffer*, SCEenum);
void* SCE_CLockVertexBuffer_ (SCEenum);
/* deverouille un tampon de sommets
   rend invalide le pointeur renvoye par LockBuffer */
void SCE_CUnlockVertexBuffer (void);

/* verouille un tampon de d'indices
   et renvoie l'adresse memoire de la zone de stockage du tampon */
void* SCE_CLockIndexBuffer (SCE_CIndexBuffer*, SCEenum);
void* SCE_CLockIndexBuffer_ (SCEenum);
/* deverouille un tampon de sommets
   rend invalide le pointeur renvoye par LockBuffer */
void SCE_CUnlockIndexBuffer (void);

/* met a jour les donnees contenues dans un tampon de sommets */
int SCE_CUpdateVertexBuffer (SCE_CVertexBuffer*);
int SCE_CUpdateVertexBuffer_ (void);
/* met a jour les donnes contenues dans un tampon d'indices */
int SCE_CUpdateIndexBuffer (SCE_CIndexBuffer*);
int SCE_CUpdateIndexBuffer_ (void);

/* selectionne un tampon de sommets
   et le defini comme tampon de sommets actif pour le rendu */
void SCE_CUseVertexBuffer (SCE_CVertexBuffer*);

/* selectionne un tampon d'indices
   et le defini comme tampon d'indices actif a utiliser pour le rendu */
void SCE_CUseIndexBuffer (SCE_CIndexBuffer*);


/* dessine le(s) tampon(s) actif(s) selectionnes pour le rendu */
void SCE_CDrawBuffer (SCEenum, GLint, GLsizei);

/* dessine le(s) tampon(s) actif(s) selectionnes pour le rendu 
   en utilisant le tampon actif d'indices*/
void SCE_CDrawIndexedBuffer (SCEenum, SCEenum, size_t, GLsizei);

#endif /* guard */
